<?php

declare(strict_types=1);

namespace Drupal\pwbi\Plugin\Field\FieldFormatter;

use Drupal\breakpoint\BreakpointManager;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\pwbi\Plugin\Field\FieldType\PowerBiEmbedField;
use Drupal\pwbi\PowerBiEmbed;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'pwbi_embed_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "pwbi_embed_formatter",
 *   label = @Translation("PowerBi Embed report"),
 *   field_types = {
 *     "pwbi_embed"
 *   }
 * )
 */
class PowerBiEmbedFormatter extends FormatterBase {

  /**
   * {@inheritDoc}
   */
  public function __construct(
    protected string $plugin_id,
    protected mixed $plugin_definition,
    protected FieldDefinitionInterface $field_definition,
    protected $settings,
    protected $label,
    protected string $view_mode,
    protected array $third_party_settings,
    protected PowerBiEmbed $pwbiEmbed,
    protected BreakpointManager $breakpointManager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    // @phpstan-ignore-next-line
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('pwbi_embed.embed'),
      $container->get('breakpoint.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $element = [];
    foreach ($items as $item) {
      $reportId = $item->getValue()['report_id'];
      $workspace = $item->getValue()['workspace_id'];
      $reportBreakpointHeight = unserialize($item->getValue()['report_breakpoints_height'], ['allowed_classes' => FALSE]);
      $reportWidth = $item->getValue()['report_width'];
      $reportHeightUnits = $item->getValue()['report_height_units'];
      $reportWidthUnits = $item->getValue()['report_width_units'];
      $reportConfiguration = $this->pwbiEmbed->getEmbedDataFromApi($workspace, $reportId);
      $tokenExpiration = new DrupalDateTime($reportConfiguration['tokenExpirationDate']);
      $cacheMaxAge = 0;
      if ($tokenExpiration->getTimestamp() > time()) {
        $cacheMaxAge = $tokenExpiration->getTimestamp() - time();
      }
      $element[] = $this->addMediaStyles($reportId, $reportBreakpointHeight, $reportHeightUnits);
      $element[] = [
        '#cache' => [
          'max-age' => $cacheMaxAge,
          'tags' => ['pwbi_embed'],
        ],
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => [
          'class' => ['pwbi-embed', 'media-' . $reportId],
          'id' => $reportId,
          'style' => ' width:' . $reportWidth . $reportWidthUnits,
        ],
        '#attached' => [
          'library' => [
            'pwbi/pwbi_client',
            'pwbi/pwbi_launcher',
          ],
          'drupalSettings' => [
            'pwbi_embed' => [
              $reportId => $this->buildEmbedConfiguration($item, $reportConfiguration),
            ],
          ],
        ],
      ];
    }
    return $element;
  }

  /**
   * Create render arrays to add media query styles.
   *
   * @param string $elementId
   *   The id of the target element.
   * @param array $heights
   *   Array with the heights for each breakpoint.
   * @param string $units
   *   The units used of the height.
   *
   * @return array
   *   Render array with the styles.
   */
  private function addMediaStyles(string $elementId, array $heights, string $units): array {
    $styles = [];
    $breakPoints = $this->breakpointManager->getBreakpointsByGroup('pwbi');
    foreach ($breakPoints as $breakPoint) {
      if (!$heights[$breakPoint->pluginId]['height']) {
        continue;
      }
      $styles[] = [
        '#type' => 'html_tag',
        '#tag' => 'style',
        '#value' => "@media " . $breakPoint->getMediaQuery() . " {
                    .media-" . $elementId . " {
                        height: " . $heights[$breakPoint->pluginId]['height'] . $units . ";
            }
                }",
      ];
    }

    return $styles;
  }

  /**
   * Create the array with the js options for embedding.
   *
   * @param \Drupal\pwbi\Plugin\Field\FieldType\PowerBiEmbedField $item
   *   Configuration option from the pwbi_embed field.
   * @param array $reportConfiguration
   *   Embed configuration from the Rest API.
   *
   * @return array
   *   The options that will be used to embed a powerbi report.
   */
  private function buildEmbedConfiguration(PowerBiEmbedField $item, array $reportConfiguration): array {
    $embedOptions = $reportConfiguration;
    $embedOptions['tokenType'] = $item->getValue()['token_type'];
    $embedOptions['type'] = $item->getValue()['embed_type'];
    $embedOptions['id'] = $item->getValue()['report_id'];
    $embedOptions['settings']['layoutType'] = (int) $item->getValue()['report_layout'];
    $embedOptions['tokenType'] = (int) ($embedOptions['tokenType'] === 'Embed');
    foreach ($embedOptions as $key => $embedOption) {
      if (empty($embedOption)) {
        unset($embedOptions[$key]);
      }
    }
    return $embedOptions;
  }

}
