<?php

declare(strict_types=1);

namespace Drupal\pwbi\Plugin\Oauth2Client;

use Drupal\oauth2_client\Plugin\Oauth2Client\Oauth2ClientPluginBase;
use Drupal\oauth2_client\Plugin\Oauth2Client\StateTokenStorage;
use League\OAuth2\Client\Provider\AbstractProvider;
use League\OAuth2\Client\Provider\GenericProvider;

/**
 * Authentication plugin for Power Bi service principal.
 *
 * @Oauth2Client(
 *   id = "pwbi_service_principal",
 *   name = @Translation("Power Bi service principal auth"),
 *   grant_type = "service_principal",
 *   label = @Translation("Power Bi service principal"),
 *   authorization_uri = "https://login.microsoftonline.com/%s/oauth2/v2.0/authorize/",
 *   token_uri = "https://login.windows.net/%s/oauth2/token/",
 *   resource_owner_uri = "https://analysis.windows.net/powerbi/api",
 *   source = "https://analysis.windows.net/powerbi/api/.default",
 * )
 */
class PwbiServicePrincipal extends Oauth2ClientPluginBase {

  use StateTokenStorage;

  /**
   * Retrieves the scope for authentication.
   *
   * @return string|null
   *   The tenant value.
   */
  public function getScope(): string|null {
    return $this->pluginDefinition['source'] ?? NULL;
  }

  /**
   * Get the credentials tenant.
   *
   * @return string
   *   The tenant value.
   */
  public function getTenant(): string {
    return $this->credentialService->getTenant($this->pluginId);
  }

  /**
   * Creates a new provider object.
   *
   * @return \League\OAuth2\Client\Provider\GenericProvider
   *   The provider of the OAuth2 Server.
   */
  public function getProvider(): AbstractProvider {
    $tenant = $this->getTenant();
    return new GenericProvider(
      [
        'clientId' => $this->getClientId(),
        'clientSecret' => $this->getClientSecret(),
        'redirectUri' => $this->getRedirectUri(),
        'urlAuthorize' => sprintf($this->getAuthorizationUri(), $tenant),
        'urlAccessToken' => sprintf($this->getTokenUri(), $tenant),
        'urlResourceOwnerDetails' => $this->getResourceUri(),
        'scopes' => $this->getScopes(),
        'scope' => $this->getScope(),
        'scopeSeparator' => $this->getScopeSeparator(),
        'tenant' => $tenant,
      ],
      $this->getCollaborators()
    );
  }

}
